@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -73,15 +73,20 @@ static CNUM table_numeric[] = {
 		{	L"Layout #1",
 			L"Layout #2",
 			L"Layout #3" } },
-	{ CFG_XTERM_TITLE,	0,
-		0, 1, 1,
-		{	L"Disabled",
-			L"Enabled" } },
 	{ CFG_MOUSE,		0,
 		0, 1, 2,
 		{	L"Disabled",
 			L"Enabled, right-handed",
 			L"Enabled, left-handed" } },
+	{ CFG_TIME_DATE,	0,
+		0, 0, 2,
+		{	L"Short format: time or date",
+			L"Long format: time and date",
+			L"Long format: date and time" } },
+	{ CFG_XTERM_TITLE,	0,
+		0, 1, 1,
+		{	L"Disabled",
+			L"Enabled" } },
 	/* really numeric */
 	{ CFG_C_SIZE,		0,			10,  120, 200 },
 	{ CFG_D_SIZE,		L"AUTO",	10,    0, 200 },
@@ -108,7 +113,6 @@ static CSTR table_string[] = {
 	{ CFG_CMD_F10,		0,	L""					},
 	{ CFG_CMD_F11,		0,	L""					},
 	{ CFG_CMD_F12,		0,	L""					},
-	{ CFG_DIR2,			L"HOME",	L""			},
 	{ CFG_FMT_TIME,		L"AUTO",	L""			},
 	{ CFG_FMT_DATE,		L"AUTO",	L""			},
 	{ CFG_LAYOUT1,		0,	L"$d $S $>$t $M $*|$p $o $L"	},
@@ -123,7 +127,7 @@ static struct {
 	CODE code;				/* internal code */
 	const char *name;		/* name in cfg file - may not exceed max length of CFGVAR_LEN */
 	const wchar_t *help;	/* help text - should fit on minimal width screen */
-} table_desc[_CFG_TOTAL_] = {
+} table_desc[CFG_TOTAL_] = {
 	{ CFG_C_SIZE,		"C_PANEL_SIZE",
 		L"Advanced: Completion panel size" },
 	{ CFG_CMD_F3,		"CMD_F3",
@@ -150,8 +154,8 @@ static struct {
 		L"Appearance: How many lines are occupied by the input line" },
 	{ CFG_D_SIZE,		"D_PANEL_SIZE",
 		L"Advanced: Directory panel size (AUTO = screen size)" },
-	{ CFG_DIR2,			"DIR2",
-		L"Secondary file panel's initial directory (HOME = home dir)" },
+	{ CFG_DOUBLE_CLICK,	"DOUBLE_CLICK",
+		L"Mouse double click interval in milliseconds" },
 	{ CFG_FRAME,		"FRAME",
 		L"Appearance: Panel frame: ----- or ===== or line graphics" },
 	{ CFG_FMT_TIME,		"TIME_FMT",
@@ -171,20 +175,20 @@ static struct {
 	{ CFG_LAYOUT3,		"LAYOUT3",
 		L"Appearance: File panel layout #3" },
 	{ CFG_MOUSE,		"MOUSE",
-		L"Mouse input (xterm compatible terminals only)" },
+		L"Mouse input (supported terminals only)" },
 	{ CFG_MOUSE_SCROLL,	"MOUSE_SCROLL",
 		L"Mouse wheel scrolls by this number of lines" },
-	{ CFG_DOUBLE_CLICK,	"DOUBLE_CLICK",
-		L"Advanced: Mouse double click interval in miliseconds" },
 	{ CFG_PROMPT,		"PROMPT",
 		L"Appearance: Command line prompt, see help" },
 	{ CFG_QUOTE,		"QUOTE",
 		L"Advanced: Additional filename chars to be quoted, see help" },
+	{ CFG_TIME_DATE,	"TIME_DATE",
+		L"Appearance: Time and date display mode" },
 	{ CFG_XTERM_TITLE,	"XTERM_TITLE",
 		L"Appearance: Change the X terminal window title" }
 };
 
-static CFG_ENTRY config[_CFG_TOTAL_];
+static CFG_ENTRY config[CFG_TOTAL_];
 
 /* 'copy' values CP_X2Y understood by set_value() */
 #define CP_SRC_CURRENT	 1
@@ -241,7 +245,7 @@ get_variable_by_name(const char *var, int len)
 {
 	int i;
 
-	for (i = 0; i < _CFG_TOTAL_; i++)
+	for (i = 0; i < CFG_TOTAL_; i++)
 		if (strncmp(table_desc[i].name,var,len) == 0)
 			return config + table_desc[i].code;
 	return 0;
@@ -274,7 +278,7 @@ get_desc(int code)
 {
 	int i;
 
-	for (i = 0; i < _CFG_TOTAL_; i++)
+	for (i = 0; i < CFG_TOTAL_; i++)
 		if (table_desc[i].code == code)
 			return i;
 	return -1;
@@ -285,6 +289,7 @@ parse_error(const char *format, ...)
 {
 	va_list argptr;
 
+	error_cnt++;
 	va_start(argptr,format);
 	vmsgout(MSG_NOTICE,format,argptr);
 	va_end(argptr);
@@ -330,6 +335,16 @@ cfgfile_read(void)
 
 		pce = get_variable_by_name(line,value - line);
 		if (pce == 0) {
+			/* --- begin 4.6 transition --- */
+			if (strncmp(line,"DIR2=",5) == 0) {
+				msgout(MSG_w,
+				  "NOTE: DIR2 is no longer a valid configuration parameter.\n"
+				  "If you want to use \"%s\" as the secondary panel's initial directory:\n"
+				  "  * please create a bookmark named DIR2 with this value\n"
+				  "  * save your configuration to purge DIR2 from the configuration file\n", value + 1);
+				continue;
+			}
+			/* --- end --- */
 			parse_error("Unknown variable in \"%s\"",line);
 			continue;
 		}
@@ -370,7 +385,7 @@ cfg_initialize(void)
 	CSTR *pstr;
 
 	/* initialize 'config' & 'pcfg' tables */
-	for (i = 0; i < _CFG_TOTAL_; i++) {
+	for (i = 0; i < CFG_TOTAL_; i++) {
 		if ( (pnum = get_numeric(i)) ) {
 			config[i].isnum = 1;
 			config[i].table = pnum;
@@ -394,7 +409,7 @@ cfg_initialize(void)
 	}
 
 	/* initialize and read values */
-	for (i = 0; i < _CFG_TOTAL_; i++)
+	for (i = 0; i < CFG_TOTAL_; i++)
 		set_value(i,CP_I2C);
 	if (cfgfile_read() < 0) {
 		if (!user_data.nowrite)
@@ -454,7 +469,7 @@ cfgfile_save(void)
 
 	if ( (fp = fw_open(user_data.file_cfg)) ) {
 		fprintf(fp,"#\n# CLEX configuration file\n#\n");
-		for (i = 0; i < _CFG_TOTAL_; i++)
+		for (i = 0; i < CFG_TOTAL_; i++)
 			if (config[i].saveit) {
 				fprintf(fp,"%s=",config[i].var);
 				if (config[i].isnum)
@@ -476,7 +491,7 @@ cfg_prepare(void)
 {
 	int i;
 
-	for (i = 0; i < _CFG_TOTAL_; i++)
+	for (i = 0; i < CFG_TOTAL_; i++)
 		set_value(i,CP_C2N);
 	panel_cfg.pd->top = panel_cfg.pd->curs = panel_cfg.pd->min;
 	panel = panel_cfg.pd;
@@ -611,7 +626,7 @@ detect_changes(void)
 	CNUM *pnum;
 	CSTR *pstr;
 
-	for (i = 0; i < _CFG_TOTAL_; i++)
+	for (i = 0; i < CFG_TOTAL_; i++)
 		if (config[i].isnum) {
 			pnum = config[i].table;
 			config[i].changed = pnum->new != pnum->current;
@@ -630,7 +645,7 @@ apply_changes(void)
 	int i;
 	FLAG reread = 0;
 
-	for (i = 0; i < _CFG_TOTAL_; i++)
+	for (i = 0; i < CFG_TOTAL_; i++)
 		if (config[i].changed)
 			set_value(i,CP_N2C);
 
@@ -646,7 +661,7 @@ apply_changes(void)
 	if (config[CFG_XTERM_TITLE].changed) {
 		xterm_title_restore();
 		xterm_title_reconfig();
-		xterm_title_set(0,0);
+		xterm_title_set(0,0,0);
 	}
 	if (config[CFG_MOUSE].changed) {
 		mouse_restore();
@@ -660,12 +675,9 @@ apply_changes(void)
 		layout_reconfig();
 		reread = 1;
 	}
-	if (config[CFG_FMT_TIME].changed) {
-		td_fmt_reconfig(TD_TIME);
-		reread = 1;
-	}
-	if (config[CFG_FMT_DATE].changed) {
-		td_fmt_reconfig(TD_DATE);
+	if (config[CFG_FMT_TIME].changed || config[CFG_FMT_DATE].changed
+	  || config[CFG_TIME_DATE].changed) {
+		td_fmt_reconfig();
 		reread = 1;
 	}
 	if (config[CFG_KILOBYTE].changed) {
