@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -61,7 +61,7 @@ enum MODE_TYPE {
 	MODE_COMPL, MODE_CMP, MODE_CMP_SUM, MODE_DESELECT,
 	MODE_DIR, MODE_DIR_SPLIT, MODE_FILE, MODE_FOPT, MODE_GROUP, MODE_HELP,
 	MODE_HIST, MODE_INSCHAR, MODE_LOG, MODE_MAINMENU, MODE_NOTIF, MODE_PASTE,
-	MODE_RENAME, MODE_SELECT, MODE_SORT, MODE_USER,
+	MODE_PREVIEW, MODE_RENAME, MODE_SELECT, MODE_SORT, MODE_USER,
 	/* pseudo-modes */
 	MODE_SPECIAL_QUIT, MODE_SPECIAL_RETURN
 };
@@ -72,11 +72,13 @@ typedef struct {
 	FLAG wait;			/* a message has been written to the
 						   text-mode screen, wait for a keypress
 						   before starting curses */
-	FLAG noenter;		/* if not null, disable "press enter to continue" after command execution */
+	FLAG noenter;		/* if set, disable "press enter to continue" after command execution */
 	unsigned int noenter_hash;		/* a hash value of the command line is saved in order to detect
 									   modifications which cancel the 'noenter' feature */
 	FLAG bs177;			/* KEY_BACKSPACE sends \177 (see control.c) */
-	FLAG xterm;			/* TERM is xterm or compatible */
+	FLAG xterm;			/* TERM is xterm or similar type which can change window title */
+	FLAG noxterm;		/* TERM is unable to change the window title */
+						/* note: if not known, both xterm and noxterm are 0 */
 	FLAG xwin;			/* running in X Window environment */
 	FLAG mouse;			/* KEY_MOUSE suitable for xterm mouse tracking */
 	FLAG mouse_swap;	/* left-handed mouse (swap left and right buttons) */
@@ -87,6 +89,7 @@ typedef struct {
 	int cmdlines;		/* number of lines in the textline area */
 	int panlines;		/* number of lines in the panel area */
 	int date_len;		/* length of date/time field */
+	int dir1end, dir2start;	/* columns of the directory names in the file panel title */
 	wchar_t *layout_panel;	/* layout: file panel part */
 	wchar_t *layout_line;	/* layout: info line part */
 } DISP_DATA;
@@ -96,7 +99,7 @@ typedef struct {
 	FLAG utf8;			/* UTF-8 mode */
 	wchar_t sep000;		/* thousands separator */
 	wchar_t repl;		/* replacement for unprintable characters */
-	const wchar_t *td_default_fmt[2];	/* time/date format strings (for strftime) */
+	const wchar_t *time_fmt, *date_fmt;	/* time/date format strings (for strftime) */
 } LANG_DATA;
 
 /* info about the user */
@@ -188,7 +191,8 @@ typedef struct {
 enum AREA_TYPE {
 	/* from top to bottom */
 	AREA_TITLE = 0, AREA_TOPFRAME, AREA_PANEL, AREA_BOTTOMFRAME,
-	AREA_INFO, AREA_HELP, AREA_BAR, AREA_LINE
+	AREA_INFO, AREA_HELP, AREA_BAR, AREA_PROMPT, AREA_LINE,
+	AREA_NONE
 };
 typedef struct {
 	/* mouse data */
@@ -199,7 +203,6 @@ typedef struct {
 
 	/* computed values */
 	int area;			/* one of AREA_xxx */
-	int xpos;			/* percentage: 0(left) to 100(right) */
 	int ypanel;			/* panel screen line (AREA_PANEL only): 0 = top */
 	int cursor;			/* calculated cursor position, see mouse_data() in inout.c */
 } MOUSE_INPUT;
@@ -207,10 +210,12 @@ typedef struct {
 /* shortcuts */
 #define MI_B(X)		(minp.button == (X))
 #define MI_DC(X)	(MI_B(X) && minp.doubleclick)
-#define MI_CLICK	(minp.button >= 1 && minp.button <= 3)
+#define MI_CLICK	(minp.button == 1 || minp.button == 3)
+#define MI_WHEEL	(minp.button == 4 || minp.button == 5)
 #define MI_AREA(X)	(minp.area == AREA_ ## X)
 #define MI_CURSBAR	(MI_AREA(PANEL) && VALID_CURSOR(panel) && panel->top + minp.ypanel == panel->curs)
 #define MI_DRAG		(minp.motion)
+#define MI_PASTE	(MI_B(3) && !MI_DRAG && MI_CURSBAR)
 
 /********************************************************************/
 
@@ -220,7 +225,7 @@ enum PANEL_TYPE {
 	PANEL_TYPE_COMPL, PANEL_TYPE_DIR, PANEL_TYPE_DIR_SPLIT, PANEL_TYPE_FILE,
 	PANEL_TYPE_FOPT, PANEL_TYPE_GROUP, PANEL_TYPE_HELP, PANEL_TYPE_HIST,
 	PANEL_TYPE_LOG, PANEL_TYPE_MAINMENU, PANEL_TYPE_NOTIF, PANEL_TYPE_PASTE,
-	PANEL_TYPE_SORT, PANEL_TYPE_USER
+	PANEL_TYPE_PREVIEW, PANEL_TYPE_SORT, PANEL_TYPE_USER
 };
 
 /*
@@ -292,10 +297,11 @@ typedef struct {
  */
 /* text buffer sizes */			/* examples: */
 #define FE_LINKS_STR	 4		/* 1  999  max  */
-#define FE_TIME_STR		12		/* 12:34am  1.01.04  01-jan-2004 */
+#define FE_TIME_STR		23		/* 11:34:30_am_2010/12/01 */
+#define FE_AGE_STR		10		/* -01:02:03 */
 #define FE_SIZE_DEV_STR	12		/* 3.222.891Ki */
 #define FE_MODE_STR 	 5		/* 0644 */
-#define FE_NAME_STR		10		/* root */
+#define FE_NAME_STR		17		/* root */
 #define FE_OWNER_STR	(2 * FE_NAME_STR)	/* root:mail */
 
 /*
@@ -328,20 +334,31 @@ typedef struct {
 	wchar_t ctime_str[FE_TIME_STR];	/* inode change time */
 	wchar_t mtime_str[FE_TIME_STR];	/* file modification time */
 	wchar_t owner_str[FE_OWNER_STR];/* owner and group */
+	char age_str[FE_AGE_STR];		/* time since the last modification ("file age") */
 	char links_str[FE_LINKS_STR];	/* number of links */
 	char mode_str[FE_MODE_STR];		/* file mode - octal number */
 	char size_str[FE_SIZE_DEV_STR];	/* file size or dev major/minor */
 } FILE_ENTRY;
 
+/*
+ * When a filter or the selection panel is activated or when panels are switched, the
+ * file panel will be refreshed if the contents are older than PANEL_EXPTIME seconds.
+ * This time is not configurable because it would confuse a typical user.
+ */
+#define PANEL_EXPTIME 60
+
 typedef struct ppanel_file {
 	PANEL_DESC *pd;
 	USTRING dir;			/* working directory */
 	USTRINGW dirw;			/* working directory for screen output */
 	struct ppanel_file *other;	/* primary <--> secondary panel ptr */
+	time_t timestamp;		/* when was the directory listed */
 	FLAG expired;			/* expiration: panel needs to be re-read */
 	FLAG filtype;			/* filter type: 0 = substring, 1 = pattern */
 	CODE order;				/* sort order: one of SORT_XXX */
 	CODE group;				/* group by type: one of GROUP_XXX */
+	CODE hide;				/* ignore hidden .files: one of HIDE_XXX */
+	FLAG hidden;			/* there exist hidden .files not shown */
 	/* unfiltered data - access only in list.c and sort.c */
 	int all_cnt;			/* number of all files */
 	int all_alloc;			/* allocated FILE_ENTRies in 'all_files' below */
@@ -358,6 +375,7 @@ typedef struct ppanel_file {
 	int cw_ln1;			/* $l */
 	int cw_sz1;			/* $r,$s,$R,$S */
 	int cw_ow1;			/* $o */
+	int cw_age;			/* $g */
 	/* field width */
 	int cw_mod;			/* $M */
 	int cw_lns;			/* $> */
@@ -390,12 +408,12 @@ typedef struct {
 /* notifications */
 enum NOTIF_TYPE {
 	NOTIF_RM = 0, NOTIF_LONG, NOTIF_DOTDIR, NOTIF_SELECTED, NOTIF_FUTURE,
-	_NOTIF_TOTAL_
+	NOTIF_TOTAL_
 };
 
 typedef struct {
 	PANEL_DESC *pd;
-	FLAG option[_NOTIF_TOTAL_];
+	FLAG option[NOTIF_TOTAL_];
 } PANEL_NOTIF;
 
 /* IMPORTANT: value 1 = notification disabled, 0 = enabled (default) */
@@ -410,12 +428,17 @@ typedef struct {
 enum SORT_TYPE {
 	SORT_NAME_NUM = 0, SORT_NAME, SORT_EXT, SORT_SIZE,
 	SORT_SIZE_REV, SORT_TIME, SORT_TIME_REV, SORT_EMAN,
-	_SORT_TOTAL_
+	SORT_TOTAL_
 };
 
 enum GROUP_TYPE {
 	GROUP_NONE = 0, GROUP_DSP, GROUP_DBCOP,
-	_GROUP_TOTAL_
+	GROUP_TOTAL_
+};
+
+enum HIDE_TYPE {
+	HIDE_NEVER, HIDE_HOME, HIDE_ALWAYS,
+	HIDE_TOTAL_
 };
 
 typedef struct {
@@ -423,9 +446,11 @@ typedef struct {
 	/* current values: */
 	CODE group;				/* group by type: one of GROUP_XXX */
 	CODE order;				/* default file sort order: one of SORT_XXX */
+	CODE hide;				/* do not show hidden .files */
 	/* for the user interface menu: */
 	CODE newgroup;
 	CODE neworder;
+	CODE newhide;
 } PANEL_SORT;
 
 /********************************************************************/
@@ -450,17 +475,18 @@ typedef struct {
 /* configuration variables in config panel order */
 enum CFG_TYPE {
 	/* appearance */
-	CFG_FRAME, CFG_CMD_LINES, CFG_XTERM_TITLE, CFG_MOUSE, CFG_MOUSE_SCROLL,
-	CFG_PROMPT, CFG_LAYOUT1, CFG_LAYOUT2, CFG_LAYOUT3, CFG_LAYOUT, CFG_KILOBYTE,
-	CFG_FMT_TIME, CFG_FMT_DATE,
+	CFG_FRAME, CFG_CMD_LINES, CFG_XTERM_TITLE, CFG_PROMPT,
+	CFG_LAYOUT1, CFG_LAYOUT2, CFG_LAYOUT3, CFG_LAYOUT, CFG_KILOBYTE,
+	CFG_FMT_TIME, CFG_FMT_DATE, CFG_TIME_DATE,
 	/* command execution */
 	CFG_CMD_F3, CFG_CMD_F4, CFG_CMD_F5, CFG_CMD_F6, CFG_CMD_F7,
 	CFG_CMD_F8, CFG_CMD_F9, CFG_CMD_F10, CFG_CMD_F11, CFG_CMD_F12,
+	/* mouse */
+	CFG_MOUSE, CFG_MOUSE_SCROLL, CFG_DOUBLE_CLICK,
 	/* other */
-	CFG_DIR2, CFG_QUOTE, CFG_DOUBLE_CLICK,
-	CFG_C_SIZE, CFG_D_SIZE, CFG_H_SIZE,
+	CFG_QUOTE, CFG_C_SIZE, CFG_D_SIZE, CFG_H_SIZE,
 	/* total count*/
-	_CFG_TOTAL_
+	CFG_TOTAL_
 };
 
 typedef struct {
@@ -538,12 +564,12 @@ typedef struct {
 /* - fopt_saveopt(), fopt_restoreopt() must remain backward compatible */
 enum FOPT_TYPE {
 	FOPT_IC, FOPT_ALL, FOPT_SHOWDIR,
-	_FOPT_TOTAL_
+	FOPT_TOTAL_
 };
 
 typedef struct {
 	PANEL_DESC *pd;
-	FLAG option[_FOPT_TOTAL_];
+	FLAG option[FOPT_TOTAL_];
 } PANEL_FOPT;
 #define FOPT(X)		(panel_fopt.option[X])
 
@@ -554,12 +580,12 @@ typedef struct {
 /* - cmp_saveopt(), cmp_restoreopt() must remain backward compatible */
 enum CMP_TYPE {
 	CMP_REGULAR, CMP_SIZE, CMP_MODE, CMP_OWNER, CMP_DATA,
-	_CMP_TOTAL_
+	CMP_TOTAL_
 };
 
 typedef struct {
 	PANEL_DESC *pd;
-	FLAG option[_CMP_TOTAL_];
+	FLAG option[CMP_TOTAL_];
 } PANEL_CMP;
 #define COPT(X)		(panel_cmp.option[X])
 
@@ -592,12 +618,24 @@ typedef struct {
 /********************************************************************/
 
 typedef struct {
-	PANEL_DESC *pd;			/* no additional data */
+	PANEL_DESC *pd;
 	FLAG wordstart;			/* complete from: 0 = beginning of the word, 1 = cursor position */
 } PANEL_PASTE;
 
 /********************************************************************/
 
+#define PREVIEW_LINES	400
+#define PREVIEW_BYTES	16383	/* fr_open() adds 1 */
+
+typedef struct {
+	PANEL_DESC *pd;
+	int realcnt;				/* lines with real data, used for --end-- mark */
+	wchar_t *title;				/* name of the file */
+	USTRINGW line[PREVIEW_LINES];
+} PANEL_PREVIEW;
+
+/********************************************************************/
+
 typedef struct {
 	uid_t uid;
 	const wchar_t *login;
@@ -608,6 +646,7 @@ typedef struct {
 	PANEL_DESC *pd;
 	USER_ENTRY *users;
 	int usr_alloc;			/* allocated entries in 'users' */
+	size_t maxlen;			/* length of the longest name */
 } PANEL_USER;
 
 typedef struct {
@@ -632,7 +671,7 @@ typedef struct {
 
 /* global variables */
 
-extern const void *pcfg[_CFG_TOTAL_];
+extern const void *pcfg[CFG_TOTAL_];
 
 extern DISP_DATA disp_data;
 extern LANG_DATA lang_data;
@@ -664,6 +703,7 @@ extern PANEL_LOG panel_log;
 extern PANEL_MENU panel_mainmenu;
 extern PANEL_NOTIF panel_notif;
 extern PANEL_PASTE panel_paste;
+extern PANEL_PREVIEW panel_preview;
 extern PANEL_SORT panel_sort;
 extern PANEL_USER panel_user;
 
