@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -70,9 +70,9 @@ const char *
 cmp_saveopt(void)
 {
 	int i, j;
-	static char buff[_CMP_TOTAL_ + 1];
+	static char buff[CMP_TOTAL_ + 1];
 
-	for (i = j = 0; i < _CMP_TOTAL_; i++)
+	for (i = j = 0; i < CMP_TOTAL_; i++)
 		if (COPT(i))
 			buff[j++] = 'A' + i;
 	buff[j] = '\0';
@@ -87,11 +87,11 @@ cmp_restoreopt(const char *opt)
 	int i;
 	unsigned char ch;
 
-	for (i = 0; i < _CMP_TOTAL_; i++)
+	for (i = 0; i < CMP_TOTAL_; i++)
 		COPT(i) = 0;
 
 	while ( (ch = *opt++) ) {
-		if (ch < 'A' || ch >= 'A' + _CMP_TOTAL_)
+		if (ch < 'A' || ch >= 'A' + CMP_TOTAL_)
 			return -1;
 		COPT(ch - 'A') = 1;
 	}
@@ -344,7 +344,7 @@ cx_cmp(void)
 	int sel;
 
 	sel = panel_cmp.pd->curs;
-	if (sel < _CMP_TOTAL_) {
+	if (sel < CMP_TOTAL_) {
 		TOGGLE(COPT(sel));
 		opt_changed();
 		win_panel_opt();
