@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -75,7 +75,7 @@ static int pd_cnt = 0;		/* number od PATHDIRs in pd_list */
 /* input: completion request data */
 static struct {
 	CODE type;			/* type of completion - one of COMPL_TYPE_XXX */
-	const wchar_t *str;	/* string to be completed */ 
+	const wchar_t *str;	/* string to be completed */
 	int strlen;			/* length of 'str' */
 	const wchar_t *dirw;/* directory to attempt the completion in */
 	const char *dir;	/* multibyte version of 'dirw' if available or 0 */
@@ -256,7 +256,7 @@ compl_prepare(void)
 	static wchar_t msg[80];	/* win_sethelp() stores this ptr */
 
 	if (compl.cnt > cc_alloc) {
-		swprintf(msg,sizeof(msg),L"%d additional entries not shown (table full)",compl.cnt - cc_alloc);
+		swprintf(msg,ARRAY_SIZE(msg),L"%d additional entries not shown (table full)",compl.cnt - cc_alloc);
 		win_sethelp(HELPMSG_TMP,msg);
 		compl.cnt = cc_alloc;
 	}
