@@ -2,27 +2,19 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
 #include "clexheaders.h"
 
-#ifdef HAVE_NCURSESW_H
-# include <ncursesw.h>
-#elif defined HAVE_NCURSES_H
-# include <ncurses.h>
-#elif defined HAVE_CURSESW_H
-# include <cursesw.h>
-#elif defined HAVE_CURSES_H
-# include <curses.h>
-#endif
+#include "curses.h"
 #include <stdarg.h>			/* log.h */
 #include <stdio.h>			/* fputs() */
 #include <stdlib.h>			/* exit() */
@@ -44,7 +36,7 @@
 #include "inout.h"			/* win_panel() */
 #include "inschar.h"		/* inschar_prepare() */
 #include "log.h"			/* vmsgout() */
-#include "mouse.h"			/* mouse_restore() */
+#include "mouse.h"			/* cx_common_mouse() */
 #include "notify.h"			/* notif_prepare() */
 #include "opt.h"			/* opt_save() */
 #include "panel.h"			/* cx_pan_xxx() */
@@ -136,6 +128,12 @@ static CXT(user,USER)
 static CXT(quit,SPECIAL_QUIT)
 static CXT(return,SPECIAL_RETURN)
 
+static void
+cx_trans_discard(void) {
+	msgout(MSG_i,"Changes discarded");
+	next_mode = MODE_SPECIAL_RETURN;
+}
+
 static void noop(void) { ; }
 
 /* defined below */
@@ -179,7 +177,7 @@ static KEY_BINDING tab_cfg[] = {
 	{ 0, 0,  L's',			cx_cfg_default,		OPT_CURS	},
 	{ 0, 0,  L'o',			cx_cfg_original,	OPT_CURS	},
 	{ 0, 1,  L'c',			cx_cfg_noexit,		0			},
-	{ 0, 0,  WCH_CTRL('C'),	cx_cfg_noexit, 		0			},
+	{ 0, 0,  WCH_CTRL('C'),	cx_trans_discard,	0			},
 	END_TABLE
 };
 
@@ -395,12 +393,12 @@ static KEY_BINDING tab_inschar[] = {
 };
 
 static KEY_BINDING tab_log[] = {
-	{ 0, 0,  WCH_CTRL('M'),	cx_pan_home,		0	},
-	{ 1, 0,  KEY_LEFT,		cx_log_left,		0	},
-	{ 1, 0,  KEY_RIGHT,		cx_log_right,		0	},
-	{ 1, 0,  KEY_HOME,		cx_log_home,		0	},
-	{ 0, 0,  L'm',			cx_log_mark,		0	},
-	{ 0, 1,  L'l',			cx_trans_return,	0	},
+	{ 0, 0,  WCH_CTRL('M'),	cx_pan_home,		0			},
+	{ 1, 0,  KEY_LEFT,		cx_log_left,		OPT_NOFILT	},
+	{ 1, 0,  KEY_RIGHT,		cx_log_right,		OPT_NOFILT	},
+	{ 1, 0,  KEY_HOME,		cx_log_home,		OPT_NOFILT	},
+	{ 0, 0,  L'm',			cx_log_mark,		OPT_NOFILT	},
+	{ 0, 1,  L'l',			cx_trans_return,	0			},
 	END_TABLE
 };
 
@@ -425,7 +423,7 @@ static KEY_BINDING tab_mainmenu[] = {
 	{ 0, 1,  L'.',			cx_files_cd_parent,	0	},
 	{ 0, 1,  L'~',			cx_files_cd_home,	0	},
 	{ 0, 1,  L'k',			cx_mode_bm,			0	},
-	{ 0, 0,  WCH_CTRL('D'),	cx_bm_addcwd,		0	}, 
+	{ 0, 0,  WCH_CTRL('D'),	cx_bm_addcwd,		0	},
 	{ 0, 1,  L'h',			cx_mode_history,	0	},
 	{ 0, 1,  L's',			cx_mode_sort,		0	},
 	{ 0, 0,  WCH_CTRL('R'),	cx_files_reread,	0	},
@@ -558,9 +556,10 @@ static KEY_BINDING tab_select[] = {
 };
 
 static KEY_BINDING tab_sort[] = {
-	{ 0, 0,  L' ',			cx_sort_set,	OPT_CURS	},
-	{ 0, 0,  WCH_CTRL('M'),	cx_sort_set,	OPT_CURS	},
-	{ 0, 1,  L's',			cx_trans_return,0			},
+	{ 0, 0,  L' ',			cx_sort_set,		OPT_CURS	},
+	{ 0, 0,  WCH_CTRL('M'),	cx_sort_set,		OPT_CURS	},
+	{ 0, 1,  L's',			cx_trans_return,	0			},
+	{ 0, 0,  WCH_CTRL('C'),	cx_trans_discard,	0			},
 	END_TABLE
 };
 
@@ -660,7 +659,7 @@ static MODE_DEFINITION mode_definition[] = {
 		group_prepare, { tab_panel,tab_group,0 } },
 	{ MODE_HELP, 0,
 		{ "help" },
-		0, L"Please report any errors or typos to clex@clex.sk",
+		0, L"Please report any errors at https://github.com/xitop/clex/issues",
 		help_prepare, { tab_help_panel,tab_panel,tab_help,0 } },
 	{ MODE_HIST, 0,
 		{ "history" },
@@ -711,12 +710,15 @@ static MODE_DEFINITION mode_definition[] = {
 };
 
 /* linked list of all control loop instances */
-static struct operation_mode {
+struct operation_mode {
 	MODE_DEFINITION *modedef;
 	PANEL_DESC *panel;
 	TEXTLINE *textline;
 	struct operation_mode *previous;
-} mode_init = { &mode_definition[ARRAY_SIZE(mode_definition) - 1],0,0,0 }, *clex_mode = &mode_init;
+};
+
+static struct operation_mode mode_init = { &mode_definition[ARRAY_SIZE(mode_definition) - 1],0,0,0 };
+static struct operation_mode *clex_mode = &mode_init;
 
 int
 get_current_mode(void)
