@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -357,8 +357,10 @@ filepos_save(void)
 			break;
 		}
 	}
-	if (new)
+	if (new) {
 		us_copy(&top->dirname,dir);
+		usw_reset(&top->dirnamew);
+	}
 
 	if (ppanel_file->pd->cnt) {
 		sd_copy(&top->savefile,SDSTR(ppanel_file->files[ppanel_file->pd->curs]->file));
@@ -435,7 +437,7 @@ cx_dir_tab(void)
 void
 cx_dir_mouse(void)
 {
-	if (textline->size == 0 && MI_B(3) && !MI_DRAG && MI_CURSBAR)
+	if (textline->size == 0 && MI_PASTE)
 		dir_paste();
 }
 
