@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -486,7 +486,7 @@ how_to_quote(wchar_t ch, int qlevel)
 	}
 	if (qlevel == QUOT_IN_QUOTES) {
 		/* inside double quotes */
-		if (ch == L'\"' || ch == L'\\' || ch == L'$')
+		if (ch == L'\"' || ch == L'\\' || ch == L'$' || ch == L'`')
 			return 1;
 	}
 	return 0;
@@ -638,7 +638,7 @@ edit_macro(const wchar_t *macro)
 	}
 
 	/* insert the rest */
-	edit_insertstr(src,0);
+	edit_insertstr(src,QUOT_NONE);
 
 	if (curs >= 0)
 		textline->curs = curs;
