@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -22,12 +22,14 @@
 
 #include "filepanel.h"
 
+#include "bookmarks.h"		/* get_bookmark() */
 #include "cfg.h"			/* cfg_str() */
 #include "completion.h"		/* compl_text() */
 #include "control.h"		/* err_exit() */
 #include "directory.h"		/* filepos_save() */
 #include "edit.h"			/* edit_macro() */
 #include "exec.h"			/* execute_cmd() */
+#include "history.h"		/* cx_hist_prev() */
 #include "inout.h"			/* win_panel() */
 #include "log.h"			/* msgout() */
 #include "list.h"			/* list_directory() */
@@ -62,7 +64,7 @@ try_all_parents(const char *directory)
 void
 files_initialize(void)
 {
-	const wchar_t *dir2;
+	const BOOKMARK *bm;
 
 	static INPUTLINE filter1, filter2;
 	static PANEL_DESC panel_desc_1 = { 0,0,0,0,PANEL_TYPE_FILE,0,0,&filter1,draw_line_file };
@@ -73,27 +75,29 @@ files_initialize(void)
 	panel_f1.other = &panel_f2;
 	ppanel_file = panel_f2.other = &panel_f1;
 
+	if ( (bm = get_bookmark(L"DIR1")) && chdir(USTR(bm->dir)) < 0)
+		msgout(MSG_w,"Bookmark DIR1: Cannot change directory: %s",strerror(errno));
 	if (us_getcwd(&panel_f1.dir) < 0) {
 		msgout(MSG_w,"Cannot get the name of the working directory (%s), it will be changed",
 		  strerror(errno));
 		try_all_parents(checkabs(getenv("PWD")) ? getenv("PWD") : user_data.homedir);
 	}
 	usw_convert2w(USTR(panel_f1.dir),&panel_f1.dirw);
-	/* do not call convert_dir() here,
-	  it could attempt to change the prompt which is not initialized yet */
+	/* do not call convert_dir() here, because
+	   it could attempt to change the prompt which is not initialized yet */
 
-	dir2 = cfg_str(CFG_DIR2);
-	if (*dir2 != L'/') {
-		if (*dir2 != L'\0')
-			msgout(MSG_NOTICE,"Ignoring the DIR2 configuration parameter, "
-			  "because it is not an absolute pathname starting with /");
-		dir2 = user_data.homedirw;	/* tested by checkabs() already */
+	if ( (bm = get_bookmark(L"DIR2")) ) {
+		usw_copy(&panel_f2.dirw,USTR(bm->dirw));
+		us_copy(&panel_f2.dir,USTR(bm->dir));
+	}
+	else {
+		usw_copy(&panel_f2.dirw,user_data.homedirw);	/* tested by checkabs() already */
+		us_convert2mb(user_data.homedirw,&panel_f2.dir);
 	}
-	usw_copy(&panel_f2.dirw,dir2);
-	us_convert2mb(dir2,&panel_f2.dir);
 
 	panel_f1.order = panel_f2.order = panel_sort.order;
 	panel_f1.group = panel_f2.group = panel_sort.group;
+	panel_f1.hide = panel_f2.hide = panel_sort.hide;
 }
 
 int
@@ -302,6 +306,8 @@ cx_files_reread_ug(void)
 void
 cx_files_xchg(void)
 {
+	int exp;
+
 	panel = (ppanel_file = ppanel_file->other)->pd;
 
 	if (chdir(USTR(ppanel_file->dir)) == -1) {
@@ -310,12 +316,10 @@ cx_files_xchg(void)
 	}
 	else {
 		update_shellprompt();	/* convert_dir() not necessary */
-		if (ppanel_file->expired)
-			list_directory();
-			/* list_directory() invokes filepos_save() */
-		else
-			/* put the new cwd to the top of the list */
-			filepos_save();
+		exp = ppanel_file->expired ? 0 : PANEL_EXPTIME;
+		if (list_directory_cond(exp) < 0)
+			/* filepanel_read() which invokes filepos_save() was not called */
+			filepos_save();		/* put the new cwd to the top of the list */
 	}
 
 	/* allow control_loop() to detect the 'panel' change */
@@ -343,6 +347,8 @@ cx_files_enter(void)
 				win_panel();
 			}
 		}
+		else if (kinp.fkey == 2 && MI_AREA(PANEL))
+			control_loop(MODE_PREVIEW);
 		else if (IS_FT_EXEC(pfe->file_type))
 			edit_macro(L"./$F ");
 	}
@@ -390,29 +396,36 @@ cx_files_mouse(void)
 {
 	int compl;
 
-	if (minp.area < AREA_BAR && MI_B(2)) {
-		control_loop(MODE_MAINMENU);
-		return;
-	}
-
-	if (MI_AREA(TITLE)) {
+	switch (minp.area) {
+	case AREA_TITLE:
 		if (MI_DC(1)) {
-			if (minp.xpos <= 50)		/* left half */
+			if (minp.x <= disp_data.dir1end)
 				control_loop(MODE_DIR);
-			else if (minp.xpos >= 66)	/* right third */
+			else if (minp.x >= disp_data.dir2start)
 				cx_files_xchg();
 		}
-		return;
-	}
-
-	if (MI_B(3) && !MI_DRAG && MI_CURSBAR) {
-		/* paste name */
-		compl = compl_text(COMPL_TYPE_DRYRUN);
-		if (compl == -1 && IS_FT_EXEC(ppanel_file->files[ppanel_file->pd->curs]->file_type))
-			edit_macro(L"./$F ");
-		else if (compl == -2)
-			edit_macro(L"$F ");
-		else
-			edit_macro(L" $F ");
+		break;
+	case AREA_PANEL:
+		if (MI_PASTE) {
+			compl = compl_text(COMPL_TYPE_DRYRUN);
+			if (compl == -1 && IS_FT_EXEC(ppanel_file->files[ppanel_file->pd->curs]->file_type))
+				edit_macro(L"./$F ");
+			else if (compl == -2)
+				edit_macro(L"$F ");
+			else
+				edit_macro(L" $F ");
+		}
+		break;
+	case AREA_BAR:
+		if (MI_DC(1) && minp.cursor == 1) {
+			control_loop(MODE_MAINMENU);
+			minp.area = AREA_NONE;		/* disable further mouse event processing */
+		}
+		break;
+	case AREA_PROMPT:
+		if (MI_DC(1))
+			control_loop(MODE_HIST);
+		else if (MI_WHEEL)
+			MI_B(4) ? cx_hist_prev() : cx_hist_next();
 	}
 }
