@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -22,6 +22,7 @@
 #include "control.h"		/* fopt_change() */
 #include "completion.h"		/* compl_panel_data() */
 #include "directory.h"		/* dir_panel_data() */
+#include "filepanel.h"		/* files_condreread() */
 #include "history.h"		/* hist_panel_data() */
 #include "inout.h"			/* win_panel() */
 #include "list.h"			/* file_panel_data() */
@@ -55,9 +56,9 @@ const char *
 fopt_saveopt(void)
 {
 	int i, j;
-	static char buff[_FOPT_TOTAL_ + 1];
+	static char buff[FOPT_TOTAL_ + 1];
 
-	for (i = j = 0; i < _FOPT_TOTAL_; i++)
+	for (i = j = 0; i < FOPT_TOTAL_; i++)
 		if (FOPT(i))
 			buff[j++] = 'A' + i;
 	buff[j] = '\0';
@@ -72,11 +73,11 @@ fopt_restoreopt(const char *opt)
 	int i;
 	unsigned char ch;
 
-	for (i = 0; i < _FOPT_TOTAL_; i++)
+	for (i = 0; i < FOPT_TOTAL_; i++)
 		FOPT(i) = 0;
 
 	while ( (ch = *opt++) ) {
-		if (ch < 'A' || ch >= 'A' + _FOPT_TOTAL_)
+		if (ch < 'A' || ch >= 'A' + FOPT_TOTAL_)
 			return -1;
 		FOPT(ch - 'A') = 1;
 	}
@@ -357,8 +358,11 @@ cx_filter(void)
 	}
 
 	if (panel->filtering == 0) {
-		if (panel->type == PANEL_TYPE_FILE)
+		if (panel->type == PANEL_TYPE_FILE) {
+			if (list_directory_cond(PANEL_EXPTIME) == 0)
+				win_panel();
 			ppanel_file->filtype = 0;
+		}
 		panel->filtering = 1;
 		cx_filteredit_kill();
 		panel->filter->changed = 0;
