@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -503,15 +503,24 @@ cx_help_down(void)
 void
 cx_help_mouse(void)
 {
-	if (MI_AREA(TITLE)) {
-		if (MI_DC(1))
-			cx_help_back();	
-	}
-	else if (MI_AREA(PANEL)) {
-		if ((MI_B(1) || MI_B(3)) && VALID_CURSOR(panel) && minp.cursor >= 0) {
+	switch (minp.area) {
+	case AREA_PANEL:
+		if (MI_CLICK && VALID_CURSOR(panel) && minp.cursor >= 0) {
 			panel_help.lnk_ln = panel->curs;
 			panel_help.lnk_act = minp.cursor;
 			win_panel_opt();
 		}
+		break;
+	case AREA_BAR:
+		if (MI_DC(1)) {
+			if (minp.cursor == 0) {
+				cx_help_main();
+				minp.area = AREA_NONE;
+			}
+			else if (minp.cursor == 2) {
+				cx_help_back();
+				minp.area = AREA_NONE;
+			}
+		}
 	}
 }
