@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -19,19 +19,11 @@
 #include <string.h>			/* strcpy() */
 #include <stdlib.h>			/* getenv() */
 #include <time.h>			/* time() */
-#include <wctype.h>			/* iswprint() */
+#include <wctype.h>			/* iswdigit() */
 #ifdef HAVE_TERM_H
-#include <term.h>			/* enter_bold_mode */
-#endif
-#ifdef HAVE_NCURSESW_H
-# include <ncursesw.h>
-#elif defined HAVE_NCURSES_H
-# include <ncurses.h>
-#elif defined HAVE_CURSESW_H
-# include <cursesw.h>
-#elif defined HAVE_CURSES_H
-# include <curses.h>
+# include <term.h>			/* enter_bold_mode */
 #endif
+#include "curses.h"
 
 #include "inout.h"
 
@@ -236,7 +228,7 @@ curses_initialize(void)
 						break;
 					}
 	}
-	msgout(MSG_DEBUG,"Terminal type: \"%s\", ability to change the window title: \"%s\"",
+	msgout(MSG_DEBUG,"Terminal type: \"%s\", can change the window title: %s",
 		term ? term : "undefined", astr);
 
 	disp_data.xwin = getenv("WINDOWID") && getenv("DISPLAY");
@@ -244,11 +236,22 @@ curses_initialize(void)
 	  disp_data.xwin ? "detected" : "not detected (no $DISPLAY and/or no $WINDOWID)");
 
 #ifdef KEY_MOUSE
-	if (key_mouse && strcmp(key_mouse,"\033[M") == 0)
+# if NCURSES_MOUSE_VERSION >= 2
+	/* version 1 does not work with the mouse wheel */
+	if (key_mouse && *key_mouse) {
+		msgout(MSG_DEBUG,"Mouse interface: ncurses mouse version %d", NCURSES_MOUSE_VERSION);
+# else
+	if (key_mouse && strcmp(key_mouse,"\033[<") == 0)
+		msgout(MSG_DEBUG,"Mouse interface: xterm SGR 1006 mode is NOT supported by CLEX");
+		/* reason: ncurses mouse is the preferred interface */
+	if (key_mouse && strcmp(key_mouse,"\033[M") == 0) {
+		msgout(MSG_DEBUG,"Mouse interface: xterm normal tracking mode");
+# endif
 		disp_data.mouse = 1;
-#endif
-	msgout(MSG_DEBUG,"Mouse tracking support: %s",
-	  disp_data.mouse ? "detected" : "not detected");
+	}
+	else
+		msgout(MSG_DEBUG,"Mouse interface: not found");
+#endif /* KEY_MOUSE */
 }
 
 /* restart CURSES */
@@ -323,7 +326,7 @@ screen_refresh(void)
 static int
 screen_area(int ln, int col)
 {
-	if (ln < 0 || ln >= disp_data.scrlines)
+	if (ln < 0 || col < 0 || ln >= disp_data.scrlines || col >= disp_data.scrcols)
 		return -1;
 	if (ln == 0)
 		return AREA_TITLE;
@@ -441,40 +444,81 @@ scr2curs_bar(int x)
 static int
 mouse_data(void)
 {
-	int mstat, mx, my;
-	int b, area, xpos;
 	FLAG click;
 	static struct timeval prev, now;
 	static MOUSE_INPUT miprev;
 
+#if NCURSES_MOUSE_VERSION >= 2
+	mmask_t mstat;
+	MEVENT mevent;
+	static CODE btn;	/* active real mouse button (i.e. not wheel) or 0 */
+
+	if (getmouse(&mevent) != OK)
+		return -1;
+	minp.x = mevent.x;
+	minp.y = mevent.y;
+	mstat = mevent.bstate;
+	minp.motion = (mstat & REPORT_MOUSE_POSITION) != 0;
+	if (minp.motion)
+		minp.button = btn;
+	else if (mstat & BUTTON1_PRESSED)
+		btn = minp.button = disp_data.mouse_swap ? 3 : 1;
+	else if (mstat & BUTTON2_PRESSED)
+		btn = minp.button = 2;
+	else if (mstat & BUTTON3_PRESSED)
+		btn = minp.button = disp_data.mouse_swap ? 1 : 3;
+	else if (mstat & BUTTON4_PRESSED)
+		minp.button = 4;
+	else if (mstat & BUTTON5_PRESSED)
+		minp.button = 5;
+	else {
+		 /* button release */
+		if (mstat & BUTTON1_RELEASED) {
+			if (btn == 1)
+				btn = 0;
+		}
+		else if (mstat & BUTTON2_RELEASED) {
+			if (btn == 2)
+				btn = 0;
+		}
+		else if (mstat & BUTTON3_RELEASED) {
+			if (btn == 3)
+				btn = 0;
+		}
+		return -1;
+	}
+
+#else
+	/* fallback: XTERM mouse */
+	int mstat;
+
 	keypad(stdscr,FALSE);
 	mstat = getch() - 32;
-	mx    = getch() - 33;
-	my    = getch() - 33;
+	minp.x = getch() - 33;
+	minp.y = getch() - 33;
 	keypad(stdscr,TRUE);
-	if (mstat < 0 || mx < 0 || my < 0)
+	if (mstat < 0)
 		return -1;
 
 	/* button */
 	switch (mstat & 0x43) {
 	case 0:
-		b = disp_data.mouse_swap ? 3 : 1; break;
+		minp.button = disp_data.mouse_swap ? 3 : 1; break;
 	case 1:
-		b = 2; break;
+		minp.button = 2; break;
 	case 2:
-		b = disp_data.mouse_swap ? 1 : 3; break;
+		minp.button = disp_data.mouse_swap ? 1 : 3; break;
 	case 64:
-		b = 4; break;
+		minp.button = 4; break;
 	case 65:
-		b = 5; break;
+		minp.button = 5; break;
 	default:
 		return -1;		/* button release or a bogus event */
 	}
+	minp.motion = mstat & 0x20; /* mouse is in motion */
+#endif
 
-	/* Y and X coordinates */
-	if ( (area = screen_area(my, mx)) < 0)
-		return -1;
-	if ((xpos = 100 * mx / (disp_data.scrcols - 1)) > 100 )
+	if ((minp.area = screen_area(minp.y, minp.x)) < 0)
 		return -1;
 
 	/* event accepted */
@@ -482,22 +526,16 @@ mouse_data(void)
 	prev = now;
 	gettimeofday(&now,0);
 
-	click = b >= 0 && b <= 3;
-	minp.button = b;
-	minp.x = mx;
-	minp.y = my;
-	minp.motion = mstat & 0x20;	/* mouse is in motion */
+	click = minp.button >= 1 && minp.button <= 3;
 	minp.doubleclick = !miprev.doubleclick && click && minp.button == miprev.button && !minp.motion
 	  && minp.x == miprev.x && minp.y == miprev.y && now.tv_sec - prev.tv_sec < 2
 	  && 1000 * (int)(now.tv_sec - prev.tv_sec) + (int)(now.tv_usec - prev.tv_usec) / 1000
 		<= cfg_num(CFG_DOUBLE_CLICK);
 
-	minp.area   = area;
-	minp.xpos   = xpos;
-	minp.ypanel = (click && area == AREA_PANEL) ? minp.y - LNO_PANEL : -1;
-	minp.cursor =  -1;
+	minp.ypanel = (click && minp.area == AREA_PANEL) ? minp.y - LNO_PANEL : -1;
+	minp.cursor = -1;
 	if (click)
-		switch (area) {
+		switch (minp.area) {
 		case AREA_LINE:
 			if (textline)
 				/* input line cursor */
@@ -629,6 +667,8 @@ ascii_code(int value)
 
 	if (lang_data.utf8 && value == L'\xAD')
 		return ", SHY (soft hyphen)";
+	if (lang_data.utf8 && value == L'\xA0')
+		return ", NBSP (non-breaking space)";
 	return (value > 0 && value < ARRAY_SIZE(ascii)) ? ascii[value] : "";
 }
 
@@ -678,7 +718,7 @@ kbd_input(void)
 	  && ((panel->filtering == 1 && (ch = panel->filter->line[panel->filter->curs]) != L'\0')
 	    || (textline != 0 && (ch = USTR(textline->line)[textline->curs]) != L'\0'))
 	  && !ISWPRINT(ch))
-		msgout(MSG_i,"unprintable character %s%s",char_code(ch),ascii_code(ch));
+		msgout(MSG_i,"special character %s%s",char_code(ch),ascii_code(ch));
 
 	kbd_getany();
 
@@ -742,7 +782,7 @@ char_line(int ch, int cnt)
 /*
  * return value:
  * if OPT_SQUEEZE is used, the return value is:
- *    -1   if the string had to be squeezed in the middle 
+ *    -1   if the string had to be squeezed in the middle
  *    >= 0 if the string was short enough to display it normally
  * if OPT_NOPAD is given, it returns the number of display columns left unused
  * otherwise it returns the number of characters written including padding,
@@ -862,6 +902,10 @@ putstr_trunc(const char *str, int maxwidth, int options)
 	return (options & OPT_NOPAD) ? remain : chcnt;
 }
 
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-but-set-variable"
+
 /* like putwcs_trunc, but stopping just before the 'endcol' screen column */
 static int
 putwcs_trunc_col(const wchar_t *str, int endcol, int options)
@@ -881,6 +925,8 @@ putstr_trunc_col(const char *str, int endcol, int options)
 	return putstr_trunc(str,endcol - x,options);
 }
 
+#pragma GCC diagnostic pop
+
 void
 win_frame_reconfig(void)
 {
@@ -1017,10 +1063,10 @@ print_position(const wchar_t *msg, int bold)
 static void
 win_position(void)
 {
-	wchar_t buffer[64];
+	wchar_t buffer[64], selected[32], *hidden;
 
 	if (posctl.resize == 2) {
-		swprintf(buffer,sizeof(buffer),L"( %dx%d )",disp_data.scrcols,disp_data.scrlines);
+		swprintf(buffer,ARRAY_SIZE(buffer),L"( %dx%d )",disp_data.scrcols,disp_data.scrlines);
 		print_position(buffer,1);
 		posctl.resize = 1;
 		return;
@@ -1048,10 +1094,13 @@ win_position(void)
 	}
 
 	if (panel->type == PANEL_TYPE_FILE && ppanel_file->selected)
-		swprintf(buffer,sizeof(buffer),L"< [%d] %d/%d >",
-		  ppanel_file->selected,panel->curs + 1,panel->cnt);
+		swprintf(selected,ARRAY_SIZE(selected),L" [%d]",ppanel_file->selected);
 	else
-		swprintf(buffer,sizeof(buffer),L"< %d/%d >",panel->curs + 1,panel->cnt);
+		selected[0] = L'\0';
+	hidden = panel->type == PANEL_TYPE_FILE && ppanel_file->hidden ? L"HIDDEN " : L"";
+
+	swprintf(buffer,ARRAY_SIZE(buffer),L"<%ls %d/%d %ls>",
+		  selected,panel->curs + 1,panel->cnt,hidden);
 	print_position(buffer,0);
 }
 
@@ -1291,10 +1340,14 @@ win_infoline(void)
         L"The mode is also known as access rights or permissions"
     },
 	*info_sort[] = {
+		0,
+		0,
+		0,
+		0,	/* ------ */
 		0,
 		L"Note: directories . and .. are always on the top, despite the sort order",
 		L"Notes: . and .. always on the top, devices sorted by device number",
-		0,
+		0,	/* ------ */
 		L"Example: file42.txt comes after file9.txt, because 42 > 9",
 		0,
 		L"The extension is also known as a file name suffix",
@@ -1302,7 +1355,8 @@ win_infoline(void)
 		L"Useful in a sendmail queue directory"
 	};
 	FILE_ENTRY *pfe;
-	const wchar_t *msg;
+	const wchar_t *msg, *ts;
+	wchar_t *pch;
 	int curs;
 
 	move(LNO_INFO,0);
@@ -1340,7 +1394,12 @@ win_infoline(void)
 			break;
 		case PANEL_TYPE_LOG:
 			putwcs_trunc(convert2w(panel_log.line[curs]->levelstr),16,0);
-			putwcs_trunc_col(convert2w(panel_log.line[curs]->timestamp),disp_data.scrcols - MARGIN2,0);
+			ts = convert2w(panel_log.line[curs]->timestamp);
+			/* some locales use non-breaking spaces, replace them in-place by regular spaces */
+			for (pch = (wchar_t *)ts; *pch; pch++)
+				if (*pch == L'\xa0')
+					*pch = L' ';
+			putwcs_trunc_col(ts,disp_data.scrcols - MARGIN2,0);
 			break;
 		case PANEL_TYPE_CMP:
 			if (curs < ARRAY_SIZE(info_cmp))
@@ -1592,7 +1651,7 @@ void
 draw_line_cmp(int ln)
 {
 	/* see also win_infoline() */
-	static const wchar_t *description[_CMP_TOTAL_ + 1] = {
+	static const wchar_t *description[CMP_TOTAL_ + 1] = {
 		L"restrict to regular files only",
 		L"compare file size",
 		L"compare file mode",
@@ -1601,7 +1660,7 @@ draw_line_cmp(int ln)
 		L"--> Compare name, type and attributes selected above"
 	};
 
-	if (ln < _CMP_TOTAL_)
+	if (ln < CMP_TOTAL_)
 		CHECKBOX(COPT(ln));	
 	putwcs_trunc_col(description[ln],disp_data.panrcol,0);
 }
@@ -1621,7 +1680,7 @@ draw_line_cmp_sum(int ln)
 	int p1, p2;
 	FLAG marked;
 
-	if (ln >= 4 && panel->cnt != sizeof(description))
+	if (ln >= 4 && panel->cnt != ARRAY_SIZE(description))
 		ln++;
 
 	txt = buf;
@@ -1630,33 +1689,33 @@ draw_line_cmp_sum(int ln)
 	case 0:
 		p1 = ppanel_file->pd->cnt        - panel_cmp_sum.nonreg1;
 		p2 = ppanel_file->other->pd->cnt - panel_cmp_sum.nonreg2;
-		swprintf(txt,sizeof(buf),L"%4d + %d%ls",p1,p2,
+		swprintf(txt,ARRAY_SIZE(buf),L"%4d + %d%ls",p1,p2,
 		  COPT(CMP_REGULAR) ? L" (regular files only)" : L"");
 		break;
 	case 1:
 		p1 = ppanel_file->pd->cnt        - panel_cmp_sum.nonreg1 - panel_cmp_sum.names;
 		p2 = ppanel_file->other->pd->cnt - panel_cmp_sum.nonreg2 - panel_cmp_sum.names;
 		if ( (marked = p1 > 0 || p2 > 0) )
-			swprintf(txt,sizeof(buf),L"%4d + %d",p1,p2);
+			swprintf(txt,ARRAY_SIZE(buf),L"%4d + %d",p1,p2);
 		else
 			txt = L"     -";
 		break;
 	case 2:
-		swprintf(txt,sizeof(buf),L"  %4d",panel_cmp_sum.names);
+		swprintf(txt,ARRAY_SIZE(buf),L"  %4d",panel_cmp_sum.names);
 		break;
 	case 3:
 		p1 = panel_cmp_sum.names - panel_cmp_sum.equal - panel_cmp_sum.errors;
 		if ( (marked = p1 > 0) )
-			swprintf(txt,sizeof(buf),L"  %4d",p1);
+			swprintf(txt,ARRAY_SIZE(buf),L"  %4d",p1);
 		else
 			txt = L"     -";
 		break;
 	case 4:
-		swprintf(txt,sizeof(buf),L"  %4d",panel_cmp_sum.errors);
+		swprintf(txt,ARRAY_SIZE(buf),L"  %4d",panel_cmp_sum.errors);
 		marked = 1;
 		break;
 	case 5:
-		swprintf(txt,sizeof(buf),L"  %4d",panel_cmp_sum.equal);
+		swprintf(txt,ARRAY_SIZE(buf),L"  %4d",panel_cmp_sum.equal);
 		break;
 	}
 	BLANK(32 - wc_cols(description[ln],0,-1));
@@ -1738,9 +1797,9 @@ draw_line_file(int ln)
 void
 draw_line_fopt(int ln)
 {
-	static const wchar_t *description[_FOPT_TOTAL_] = {
+	static const wchar_t *description[FOPT_TOTAL_] = {
 		L"substring matching: ignore the case of the characters",
-		L"pattern matching: wildcards match the dot in .hidden files",
+		L"pattern matching: wildcards match the dot in hidden .files",
 		L"file panel filtering: always show directories",
 		/* must correspond with FOPT_XXX */
 	};
@@ -1854,12 +1913,12 @@ draw_line_mainmenu(int ln)
 void
 draw_line_notif(int ln)
 {
-	static const wchar_t *description[_NOTIF_TOTAL_] = {
+	static const wchar_t *description[NOTIF_TOTAL_] = {
 		L"Warning:  rm command deletes files (not 100% reliable)",
 		L"Warning:  command line is too long to be displayed",
 		L"Reminder: selection marks on . and .. are not honored",
 		L"Reminder: selected file(s) vs. current file",
-		L"Notice:   file with a timestamp in the future encountered" 
+		L"Notice:   file with a timestamp in the future encountered"
 		/* must correspond with NOPT_XXX */
 	};
 
@@ -1912,16 +1971,23 @@ draw_line_sort(int ln)
 	int size;
 
 	/* see also win_infoline() */
-	static const wchar_t *description1[_GROUP_TOTAL_] = {
+	static const wchar_t
+	*description0[HIDE_TOTAL_] = {
+		L"show hidden .files",
+		L"show hidden .files, but not in the home directory",
+		L"do not show hidden .files"
+		/* must correspond with HIDE_XXX */
+	},
+	*description1[GROUP_TOTAL_] = {
 		L"do not group files by type",
 		L"group: directories, special files, plain files",
 		L"group: directories, devices, special files, plain files"
 		/* must correspond with GROUP_XXX */
 	},
-	*description2[_SORT_TOTAL_] = {
+	*description2[SORT_TOTAL_] = {
 		L"sort by name and number",
 		L"sort by name",
-		L"sort by .extension",
+		L"sort by filename.EXTENSION",
 		L"sort by size [small -> large]",
 		L"sort by size [large -> small]",
 		L"sort by time of last modification [recent -> old]",
@@ -1930,10 +1996,22 @@ draw_line_sort(int ln)
 		/* must correspond with SORT_XXX */
 	},
 	*description3[3] = {
-		L"--> apply to ALL DIRECTORIES",
-		L"--> apply to this panel and THIS DIRECTORY ONLY"
+		L"--> save & apply globally",
+		L"--> apply temporarily to the current file panel's contents"
 	};
 
+	size = ARRAY_SIZE(description0);
+	if (ln < size) {
+		RADIOBUTTON(panel_sort.newhide == ln);
+		putwcs_trunc(description0[ln],disp_data.pancols - BOX4,0);
+		return;
+	}
+	if (ln == size) {
+		putstr_trunc("----------------",disp_data.pancols,0);
+		return;
+	}
+	ln -= size + 1;
+
 	size = ARRAY_SIZE(description1);
 	if (ln < size) {
 		RADIOBUTTON(panel_sort.newgroup == ln);
@@ -1958,11 +2036,18 @@ draw_line_sort(int ln)
 	return;
 }
 
+#define MIN_GECOS 10	/* columns reserved for the gecos field */
+
 void
 draw_line_user(int ln)
 {
+	int col;
+
 	printw("%6u  ",(unsigned int)panel_user.users[ln].uid);
-	putwcs_trunc_col(panel_user.users[ln].login,24,0);
+	col = MARGIN2 + 8 /*printw above*/ + panel_user.maxlen;
+	if (col > disp_data.panrcol - MIN_GECOS - 1 /* 1 space */)
+		col = disp_data.panrcol - MIN_GECOS - 1;
+	putwcs_trunc_col(panel_user.users[ln].login,MARGIN2 + 8 + panel_user.maxlen,0);
 	addch(' ');
 	putwcs_trunc_col(panel_user.users[ln].gecos,disp_data.panrcol,0);
 }
