@@ -2,34 +2,29 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
 #include "../config.h"
 
+#include "clexheaders.h"
+
 #include <locale.h>
 #include <stdlib.h>
 #include <time.h>
 #include <wchar.h>
 #include <wctype.h>
 
-#ifdef HAVE_NCURSESW_H
-# include <ncursesw.h>
-#elif defined HAVE_NCURSES_H
-# include <ncurses.h>
-#elif defined HAVE_CURSESW_H
-# include <cursesw.h>
-#elif defined HAVE_CURSES_H
-# include <curses.h>
-#endif
+#include "curses.h"
 
+#pragma GCC diagnostic ignored "-Wunused-but-set-variable"
 int
 main(void)
 {
