@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -21,7 +21,6 @@
 
 #include "lang.h"
 
-#include "list.h"		/* TD_xxx */
 #include "log.h"		/* msgout() */
 #include "mbwstring.h"	/* convert2w() */
 #include "util.h"		/* ewcsdup() */
@@ -65,7 +64,7 @@ locale_initialize(void)
 	lang_data.sep000 = sep000();
 	tf = nl_langinfo(T_FMT);
 	df = nl_langinfo(D_FMT);
-	lang_data.td_default_fmt[TD_TIME] = ewcsdup(convert2w(tf));
-	lang_data.td_default_fmt[TD_DATE] = ewcsdup(convert2w(df));
+	lang_data.time_fmt = ewcsdup(convert2w(tf));
+	lang_data.date_fmt = ewcsdup(convert2w(df));
 	msgout(MSG_DEBUG,"LOCALE: standard time format: \"%s\", standard date format: \"%s\"",tf,df);
 }
