@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -65,8 +65,8 @@
 static int ucache_cnt = 0, gcache_cnt = 0;
 
 /* layout */
-static FLAG do_gt, do_a, do_d, do_i, do_l, do_L, do_m, do_m_blank;	/* fields in use */
-static FLAG do_o, do_s, do_s_short, do_s_nodir;
+static FLAG do_gt, do_a, do_d, do_i, do_l, do_L, do_g,
+	do_m, do_m_blank, do_o, do_s, do_s_short, do_s_nodir;		/* fields in use */
 static mode_t normal_file, normal_dir;
 
 /* kilobyte definition */
@@ -76,13 +76,20 @@ static int K2, K995;				/* kilobyte/2, kilobyte*9.95 */
 static time_t now;					/* current time */
 static int now_day;					/* current day of month */
 static struct tm testtm;			/* tm struct for format string checks */
-static const wchar_t *td_fmt[2];	/* time/date format (man strftime, man wcsftime) */
-static int tdpad[2], tdlen[2];		/* time/date lengths: padding + output string */
-					/* tdpad[0] + tdlen[0] = tdpad[1] + tdlen[1] = disp_data.date_len */
-static const wchar_t *td_failsafe[2] =
-  { L"%H:%M", L"%Y-%m-%d" }; 		/* failsafe time/date format */
-static FLAG td_fmt_fail[2];			/* time/date output format failed */
 static FLAG future;
+static FLAG td_both;				/* both time and date are displayed */
+#define TD_TIME	0
+#define TD_DATE	1
+#define TD_BOTH	2
+static const wchar_t *td_fmt[3];	/* time/date formats (man strftime, man wcsftime) */
+static int td_pad[3], td_len[3];	/* time/date padding + output string lengths */
+									/* padding + length = disp_data.date_len */
+static const wchar_t *td_fmt_fail;	/* failed time/date output format */
+
+#define FAILSAFE_TIME		L"%H:%M"
+#define FAILSAFE_DATE		L"%Y-%m-%d"
+#define FAILSAFE_TIMEDATE	L"%H:%M %Y-%m-%d"
+#define FAILSAFE_DATETIME	L"%Y-%m-%d %H:%M"
 
 /* directory listing */
 static FLAG mm_change;				/* minor/major format changed during the operation */
@@ -105,24 +112,19 @@ check_str(const wchar_t *str)
 
 /* return value: output string length if it does fit, -1 otherwise */
 static int
-check_format(int td)
+check_format(const wchar_t *format)
 {
 	int len;
-	const wchar_t *format;
 	wchar_t tmp[80];
 
-	format = td_fmt[td];
-
 	if (check_str(format) < 0) {
-		td_fmt_fail[td] = 1;
 		msgout(MSG_NOTICE,"Date/time output format \"%ls\" rejected "
 		  "because of non-standard characters",format);
 		return -1;
 	}
 
-	len = wcsftime(tmp,sizeof(tmp),format,&testtm);
+	len = wcsftime(tmp,ARRAY_SIZE(tmp),format,&testtm);
 	if (len == 0 || len >= FE_TIME_STR) {
-		td_fmt_fail[td] = 1;
 		msgout(MSG_NOTICE,"Date/time output format \"%ls\" rejected "
 		  "because it produces output longer than a limit of %d characters",
 		  format,FE_TIME_STR - 1);
@@ -131,36 +133,68 @@ check_format(int td)
 	return len;
 }
 
-/* td = TD_xxx */
 void
-td_fmt_reconfig(int td)
+td_fmt_reconfig(void)
 {
-	int len, min, max;
+	int order, fmt_fail, len, diff;
+	const wchar_t *fmt;
+	static USTRINGW buff = UNULL;
 
-	td_fmt_fail[td] = 0;
+	fmt_fail = 0;
 
-	td_fmt[td] = cfg_str(td == TD_TIME ? CFG_FMT_TIME : CFG_FMT_DATE);
-	if (*td_fmt[td] != L'\0' && (len = check_format(td)) > 0)
-		/* customized format accepted */;
-	else if (td_fmt[td] = lang_data.td_default_fmt[td], (len = check_format(td)) > 0)
-		/* default format accepted */;
-	else if (td_fmt[td] = td_failsafe[td], (len = check_format(td)) > 0)
-		/* using failsafe format */;
-	else
-		err_exit("BUG: Failsafe date/time output format failed");
-	tdlen[td] = len;
+	fmt = cfg_str(CFG_FMT_TIME);
+	if (*fmt == L'\0')
+		fmt = lang_data.time_fmt;
+	if ((len = check_format(fmt)) < 0) {
+		fmt_fail = 1;
+		fmt = FAILSAFE_TIME;
+		len = check_format(fmt);
+	}
+	td_fmt[TD_TIME] = fmt;
+	td_pad[TD_TIME] = 0;
+	td_len[TD_TIME] = len;
 
-	/* calculate padding */
-	/* input: tdlen[];  output: tdpad[] and disp_data.date_len */
-	max = tdlen[0] < tdlen[1];	/* 0 or 1 */
-	min = 1 - max;				/* 1 or 0 */
-	disp_data.date_len = tdlen[max];
-	tdpad[max] = 0;
-	tdpad[min] = tdlen[max] - tdlen[min];
+	fmt = cfg_str(CFG_FMT_DATE);
+	if (*fmt == L'\0')
+		fmt = lang_data.date_fmt;
+	if ((len = check_format(fmt)) < 0) {
+		fmt_fail = 1;
+		fmt = FAILSAFE_DATE;
+		len = check_format(fmt);
+	}
+	td_fmt[TD_DATE] = fmt;
+	td_pad[TD_DATE] = 0;
+	td_len[TD_DATE] = len;
 
-	if (td_fmt_fail[td])
-		msgout(MSG_w,"Problem with %s output format, details in log",
-		  td == TD_TIME ? "time" : "date");
+	td_both = cfg_num(CFG_TIME_DATE) != 0;
+	if (td_both) {
+		order = cfg_num(CFG_TIME_DATE) == 1;
+		usw_cat(&buff, td_fmt[order ? TD_TIME : TD_DATE], L" ",
+		  td_fmt[order ? TD_DATE : TD_TIME], (wchar_t *)0);
+		fmt = USTR(buff);
+		if ((len = check_format(fmt)) < 0) {
+			fmt_fail = 1;
+			fmt = order ? FAILSAFE_TIMEDATE : FAILSAFE_DATETIME;
+			len = check_format(fmt);
+		}
+		td_fmt[TD_BOTH] = fmt;
+		td_pad[TD_BOTH] = 0;
+		disp_data.date_len = td_len[TD_BOTH] = len;
+	}
+	else {
+		diff = td_len[TD_DATE] - td_len[TD_TIME];
+		if (diff > 0) {
+			td_pad[TD_TIME] = diff;
+			disp_data.date_len = td_len[TD_DATE];
+		}
+		else {
+			td_pad[TD_DATE] = -diff;
+			disp_data.date_len = td_len[TD_TIME];
+		}
+	}
+
+	if (fmt_fail)
+		msgout(MSG_w,"Problem with time/date output format, details in log");
 }
 
 void
@@ -211,7 +245,7 @@ layout_reconfig(void)
 	split_layout();
 
 	/* which fields are going to be displayed ? */
-	do_gt = do_a = do_d = do_i = do_L = do_l = 0;
+	do_gt = do_a = do_d = do_i = do_g = do_L = do_l = 0;
 	do_m = do_m_blank = do_o = do_s = do_s_short = do_s_nodir = 0;
 	for (field = 0, layout = cfg_layout /*macro*/; (ch = *layout++); )
 		if (!TCLR(field)) {
@@ -223,6 +257,7 @@ layout_reconfig(void)
 			case L'>': do_gt = 1; break;
 			case L'a': do_a = 1; break;
 			case L'd': do_d = 1; break;
+			case L'g': do_g = 1; break;
 			case L'i': do_i = 1; break;
 			case L'L': do_L = 1; break;
 			case L'l': do_l = 1; break;
@@ -253,8 +288,7 @@ list_initialize(void)
 
 	kb_reconfig();
 	layout_reconfig();
-	td_fmt_reconfig(TD_TIME);
-	td_fmt_reconfig(TD_DATE);
+	td_fmt_reconfig();
 }
 
 /*
@@ -268,7 +302,9 @@ stat2time(wchar_t *str, time_t tm)
 {
 	int i, td, len;
 
-	if (tm <= now) {
+	if (td_both)
+		td = TD_BOTH;
+	else if (tm <= now) {
 		if (tm < now - 86400 /* more than 1 day ago */)
 			td = TD_DATE;
 		else if (tm > now - 57600 /* less than 16 hours ago */)
@@ -282,28 +318,59 @@ stat2time(wchar_t *str, time_t tm)
 		td = (tm > now + 86400 || localtime(&tm)->tm_mday != now_day) ? TD_DATE: TD_TIME;
 	}
 	
-	for (i = 0; i < tdpad[td]; i++)
+	for (i = 0; i < td_pad[td]; i++)
 		*str++ = L' ';
-	len = wcsftime(str,FE_TIME_STR - tdpad[td],td_fmt[td],localtime(&tm));
-	if (len == tdlen[td])
+	len = wcsftime(str,FE_TIME_STR - td_pad[td],td_fmt[td],localtime(&tm));
+	if (len == td_len[td])
 		return;
 
 	/* problem with the format: output length is not constant */
-	if (len > 0 && len < tdlen[td]) {
+	if (len > 0 && len < td_len[td]) {
 		/* small problem: the string is shorter */
-		for (i = len; i < tdlen[td]; i++)
+		for (i = len; i < td_len[td]; i++)
 			str[i] = L' ';
 		str[i] = L'\0';
 	}
 	else {
 		/* big problem: the string does not fit */
-		td_fmt_fail[td] = 1;
-		for (i = 0; i < tdlen[td]; i++)
+		td_fmt_fail = td_fmt[td];
+		for (i = 0; i < td_len[td]; i++)
 			str[i] = L'-';
 		str[i] = L'\0';
 	}
 }
 
+static void
+stat2age(char *str, time_t tm)
+{
+	time_t age;
+	int h, m, s;
+
+	age = now - tm;
+
+	if (age < 0) {
+		strcpy(str,"  future!");
+		return;
+	}
+	if (age >= 360000 /* 100 hours */) {
+		strcpy(str,"         ");
+		return;
+	}
+
+	h = age / 3600;
+	age -= h * 3600;
+	m = age / 60;
+	s = age - m * 60;
+	if (h)
+		sprintf(str,"%3d:%02d:%02d",-h,m,s);
+	else if (m)
+		sprintf(str,"   %3d:%02d",-m,s);
+	else if (s)
+		sprintf(str,"      %3d",-s);
+	else
+		strcpy(str,"       -0");
+}
+
 static void
 stat2size_7(char *str, off_t size)
 {
@@ -361,7 +428,7 @@ stat2size_3(char *str, off_t size)
 
 /*
  * stat2dev() prints device major:minor numbers
- * 
+ *
  * FE_SIZE_DEV_STR is 12 ==> total number of digits is 10,
  * from these 10 digits are 2 to 7 used for minor device
  * number (printed in hex) and the rest is used for major
@@ -466,7 +533,7 @@ id2name(wchar_t *dst, int leftalign, const wchar_t *name, unsigned int id)
 	wchar_t number[16];
 
 	if (check_str(name) < 0) {
-		swprintf(number,sizeof(number),L"%u",id);
+		swprintf(number,ARRAY_SIZE(number),L"%u",id);
 		name = number;
 	}
 
@@ -594,6 +661,7 @@ nofileinfo(FILE_ENTRY *pfe)
 	pfe->mtime_str[0] = L'\0';
 	pfe->ctime_str[0] = L'\0';
 	pfe->links_str[0] = '\0';
+	pfe->age_str[0] = '\0';
 	pfe->links = 0;
 	pfe->mode_str[0] = '\0';
 	pfe->normal_mode = 1;
@@ -623,6 +691,8 @@ fileinfo(FILE_ENTRY *pfe, struct stat *pst)
 		stat2time(pfe->atime_str,pst->st_atime);
 	if (do_d)
 		stat2time(pfe->mtime_str,pst->st_mtime);
+	if (do_g)
+		stat2age(pfe->age_str,pst->st_mtime);
 	if (do_i)
 		stat2time(pfe->ctime_str,pst->st_ctime);
 	if (do_l)
@@ -662,12 +732,13 @@ fileinfo(FILE_ENTRY *pfe, struct stat *pst)
 static void
 set_cw(void)
 {
-	int i, mod, lns, lnh, ln1, sz1, ow1, sz2, ow2;
+	int i, mod, lns, lnh, ln1, sz1, ow1, sz2, ow2, age;
 	FILE_ENTRY *pfe;
 
 	mod = do_m_blank;
 	lns = do_gt;
 	lnh = do_L;
+	age = FE_AGE_STR - 2;
 	ln1 = FE_LINKS_STR - 3;
 	sz1 = FE_SIZE_DEV_STR - 4;
 	ow1 = FE_NAME_STR - 3;
@@ -682,6 +753,9 @@ set_cw(void)
 			lns = 0;
 		if (lnh && pfe->links)
 			lnh = 0;
+		if (do_g && *pfe->age_str)
+			while (age >= 0 && pfe->age_str[age] != ' ')
+				age--;
 		if (do_l && *pfe->links_str)
 			while (ln1 >= 0 && pfe->links_str[ln1] != ' ')
 				ln1--;
@@ -713,6 +787,7 @@ set_cw(void)
 	ppanel_file->cw_ln1 = ln1 + 1;
 	ppanel_file->cw_sz1 = sz1 + 1;
 	ppanel_file->cw_ow1 = ow1 + 1;
+	ppanel_file->cw_age = age + 1;
 	ppanel_file->cw_sz2 = sz2 - sz1 - 1;
 	ppanel_file->cw_ow2 = ow2 - ow1 - 1;
 }
@@ -752,12 +827,21 @@ describe_file(const char *name, FILE_ENTRY *pfe)
 	return 0;
 }
 
-static void
-log_format_error(int td)
+#define DOT_NONE		0	/* not a .file */
+#define DOT_DIR			1	/* dot directory */
+#define DOT_DOT_DIR		2	/* dot-dot directory */
+#define DOT_HIDDEN		3	/* hidden .file */
+
+static int
+dotfile(const char *name)
 {
-	msgout(MSG_NOTICE,"Time/date output format \"%ls\" produces output of variable length, "
-	  "check the configuration parameter \"%s\"",
-	  td == TD_TIME ? "TIME_FMT" : "DATE_FMT",td_fmt[td]);
+	if (name[0] != '.')
+		return DOT_NONE;
+	if (name[1] == '\0')
+		return DOT_DIR;
+	if (name[1] == '.' && name[2] == '\0')
+		return DOT_DOT_DIR;
+	return DOT_HIDDEN;
 }
 
 static void
@@ -766,6 +850,7 @@ directory_read(void)
 	int i, cnt1, cnt2;
 	DIR *dd;
 	FILE_ENTRY *pfe;
+	FLAG hide;
 	struct stat st;
 	struct dirent *direntry;
 	const char *name;
@@ -780,7 +865,12 @@ directory_read(void)
 	dirdev = st.st_dev;
 
 	win_waitmsg();
-	mm_change = td_fmt_fail[0] = td_fmt_fail[1] = future = 0;
+	mm_change = future = 0;
+	td_fmt_fail = 0;
+	ppanel_file->hidden = 0;
+	hide = ppanel_file->hide == HIDE_ALWAYS
+		   || (ppanel_file->hide == HIDE_HOME
+			   && strcmp(USTR(ppanel_file->dir),user_data.homedir) == 0);
 
 	/*
 	 * step #1: process selected files already listed in the panel
@@ -794,7 +884,7 @@ directory_read(void)
 		if (!pfe->select)
 			continue;
 		name = SDSTR(pfe->file);
-		if (describe_file(name, pfe) < 0)
+		if ((hide && dotfile(name) == DOT_HIDDEN) || describe_file(name, pfe) < 0)
 			/* this entry is no more valid */
 			ppanel_file->selected--;
 		else {
@@ -810,8 +900,12 @@ directory_read(void)
 	cnt2 = cnt1;
 	while ( (direntry = readdir(dd)) ) {
 		name = direntry->d_name;
+		if (hide && dotfile(name) == DOT_HIDDEN) {
+			ppanel_file->hidden = 1;
+			continue;
+		}
 
-		/* didn't we see this file already in step #1 ? */
+		/* didn't we process this file already in step #1 ? */
 		if (cnt1) {
 			for (i = 0; i < cnt1; i++)
 				if (strcmp(SDSTR(ppanel_file->all_files[i]->file),name) == 0)
@@ -838,13 +932,9 @@ directory_read(void)
 		pfe = ppanel_file->all_files[cnt2];
 		sd_copy(&pfe->file,name);
 		sdw_copy(&pfe->filew,convert2w(name));
-		pfe->dotdir = 0;
-		if (name[0] == '.') {
-			if (name[1] == '\0')
-				pfe->dotdir = 1;
-			else if (name[1] == '.' && name[2] == '\0')
-				pfe->dotdir = 2;
-		}
+		pfe->dotdir = dotfile(name);
+		if (pfe->dotdir == DOT_HIDDEN)
+			pfe->dotdir = DOT_NONE;
 		if (describe_file(name, pfe) < 0)
 			continue;
 		pfe->select = 0;
@@ -861,11 +951,9 @@ directory_read(void)
 				stat2dev(pfe->size_str,major(pfe->devnum),minor(pfe->devnum));
 		}
 
-	if (td_fmt_fail[TD_TIME] || td_fmt_fail[TD_DATE]) {
-		if (td_fmt_fail[TD_TIME])
-			log_format_error(TD_TIME);
-		if (td_fmt_fail[TD_DATE])
-			log_format_error(TD_DATE);
+	if (td_fmt_fail) {
+		msgout(MSG_NOTICE,"Time/date format \"%ls\" produces output of variable length, "
+		  "check the configuration",td_fmt_fail);
 		msgout(MSG_w,"Problem with date/time output format, details in log");
 	}
 	if (future && !NOPT(NOTIF_FUTURE))
@@ -882,6 +970,7 @@ filepanel_reset(void)
 	ppanel_file->selected = ppanel_file->selected_out = 0;
 	ppanel_file->order = panel_sort.order;
 	ppanel_file->group = panel_sort.group;
+	ppanel_file->hide = panel_sort.hide;
 }
 
 void
@@ -973,22 +1062,32 @@ filepanel_read(void)
 	sort_files();
 	/* sort_files() calls file_panel_data() */
 	filepos_set();
+	ppanel_file->timestamp = now;
 	ppanel_file->expired = 0;
 }
 
-void
-list_directory(void)
+int
+list_directory_cond(int expiration_time)
 {
 	now = time(0);
-	now_day = localtime(&now)->tm_mday;
 
 	/* password data change invalidates data in both panels */
 	if (userdata_refresh()) {
 		ppanel_file->other->expired = 1;
 		ucache_cnt = gcache_cnt = 0;
 	}
+	else if (expiration_time && now < ppanel_file->timestamp + expiration_time)
+		return -1;
 
+	now_day = localtime(&now)->tm_mday;
 	filepanel_read();
+	return 0;
+}
+
+void
+list_directory(void)
+{
+	list_directory_cond(0);
 }
 
 void
