@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -33,9 +33,9 @@ const char *
 notif_saveopt(void)
 {
 	int i, j;
-	static char buff[_NOTIF_TOTAL_ + 1];
+	static char buff[NOTIF_TOTAL_ + 1];
 
-	for (i = j = 0; i < _NOTIF_TOTAL_; i++)
+	for (i = j = 0; i < NOTIF_TOTAL_; i++)
 		if (NOPT(i))
 			buff[j++] = 'A' + i;
 	buff[j] = '\0';
@@ -50,11 +50,11 @@ notif_restoreopt(const char *opt)
 	int i;
 	unsigned char ch;
 
-	for (i = 0; i < _NOTIF_TOTAL_; i++)
+	for (i = 0; i < NOTIF_TOTAL_; i++)
 		NOPT(i) = 0;
 
 	while ( (ch = *opt++) ) {
-		if (ch < 'A' || ch >= 'A' + _NOTIF_TOTAL_)
+		if (ch < 'A' || ch >= 'A' + NOTIF_TOTAL_)
 			return -1;
 		NOPT(ch - 'A') = 1;
 	}
