@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -85,17 +85,14 @@ cx_pan_mouse(void)
 		else
 			(MI_B(4) ? pan_up_n : pan_down_n)(cfg_num(CFG_MOUSE_SCROLL));
 		break;
+	case AREA_TOPFRAME:
+		if (MI_CLICK)
+			cx_pan_pgup();
+		break;
 	case AREA_BOTTOMFRAME:
-		if ((panel->filtering == 0 && MI_DC(1)) || panel->filtering == 2)
-			cx_filter();
-		if (panel->filtering != 1 || panel->filter->size == 0)
-			return;
-		if (MI_CLICK) {
-			if (minp.cursor >= 0)
-				panel->filter->curs = minp.cursor;
-		}
-		else
-			MI_B(4) ? cx_filteredit_left() : cx_filteredit_right();
+		if (MI_CLICK)
+			cx_pan_pgdown();
+		break;
 	}
 }
 
