@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -51,7 +51,7 @@ rename_prepare(void)
 	textline = &line_tmp;
 	edit_nu_putstr(SDSTR(pfe->filew));
 	for (pch = USTR(textline->line); (ch = *pch) != L'\0'; pch++)
-		if (!ISWPRINT(ch) || ch == L' ' || (lang_data.utf8 && ch == L'\xFFFD'))
+		if (!ISWPRINT(ch) || (lang_data.utf8 && ch == L'\xFFFD'))
 			*pch = L'_';
 
 	return 0;
