@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -37,9 +37,11 @@ int
 sort_prepare(void)
 {
 	panel_sort.pd->top = panel_sort.pd->min;
-	panel_sort.pd->curs = 1 + _GROUP_TOTAL_ + panel_sort.order;
+	panel_sort.pd->curs = HIDE_TOTAL_ + GROUP_TOTAL_ + 2 + panel_sort.order;
+						  /* place the cursor at the current sort order */
 	panel_sort.newgroup = panel_sort.group;
 	panel_sort.neworder = panel_sort.order;
+	panel_sort.newhide = panel_sort.hide;
 	panel = panel_sort.pd;
 	textline = 0;
 	return 0;
@@ -52,17 +54,26 @@ cx_sort_set(void)
 
 	sel = panel_sort.pd->curs;
 
-	if (sel < _GROUP_TOTAL_) {
+	if (sel < HIDE_TOTAL_) {
+		panel_sort.newhide = sel;
+		win_panel();
+		return;
+	}
+	if (sel == HIDE_TOTAL_)
+		return;
+	sel -= HIDE_TOTAL_ + 1;
+
+	if (sel < GROUP_TOTAL_) {
 		/* set grouping */
 		panel_sort.newgroup = sel;
 		win_panel();
 		return;
 	}
-	if (sel == _GROUP_TOTAL_)
+	if (sel == GROUP_TOTAL_)
 		return;
+	sel -= GROUP_TOTAL_ + 1;
 
-	sel -= _GROUP_TOTAL_ + 1;
-	if (sel < _SORT_TOTAL_) {
+	if (sel < SORT_TOTAL_) {
 		/* set sort order */
 		panel_sort.neworder = sel;
 		win_panel();
@@ -70,23 +81,33 @@ cx_sort_set(void)
 	}
 
 	/* activate settings */
-	sel -= _SORT_TOTAL_;
+	sel -= SORT_TOTAL_;
 	if (sel == 0) {
 		/* setting the default */
 		if (panel_sort.order != panel_sort.neworder
-		  || panel_sort.group != panel_sort.newgroup) {
+		  || panel_sort.group != panel_sort.newgroup
+		  || panel_sort.hide != panel_sort.newhide) {
 			panel_sort.group = panel_sort.newgroup;
 			panel_sort.order = panel_sort.neworder;
+			panel_sort.hide = panel_sort.newhide;
 			opt_changed();
 		}
 		if (ppanel_file->other->order != panel_sort.neworder
-		  || ppanel_file->other->group != panel_sort.newgroup) {
+		  || ppanel_file->other->group != panel_sort.newgroup
+		  || ppanel_file->other->hide != panel_sort.newhide) {
 			ppanel_file->other->group = panel_sort.newgroup;
 			ppanel_file->other->order = panel_sort.neworder;
+			ppanel_file->other->hide = panel_sort.newhide;
 			ppanel_file->other->expired = 1;
 		}
 	}
-	if (ppanel_file->order != panel_sort.neworder
+	if (ppanel_file->hide != panel_sort.newhide) {
+		ppanel_file->group = panel_sort.newgroup;
+		ppanel_file->order = panel_sort.neworder;
+		ppanel_file->hide = panel_sort.newhide;
+		list_directory();
+	}
+	else if (ppanel_file->order != panel_sort.neworder
 	  || ppanel_file->group != panel_sort.newgroup) {
 		ppanel_file->group = panel_sort.newgroup;
 		ppanel_file->order = panel_sort.neworder;
@@ -100,10 +121,11 @@ cx_sort_set(void)
 const char *
 sort_saveopt(void)
 {
-	static char buff[3] = "??";
+	static char buff[4] = "???";
 
 	buff[0] = 'A' + panel_sort.order;
 	buff[1] = 'A' + panel_sort.group;
+	buff[2] = 'A' + panel_sort.hide;
 
 	return buff;
 }
@@ -114,18 +136,21 @@ sort_restoreopt(const char *opt)
 {
 	unsigned char ch;
 
-	panel_sort.order = SORT_NAME;
-	panel_sort.group = GROUP_DSP;
-
 	ch = opt[0];
-	if (ch < 'A' || ch >= 'A' + _SORT_TOTAL_)
+	if (ch < 'A' || ch >= 'A' + SORT_TOTAL_)
 		return -1;
 	panel_sort.order = ch - 'A';
 	ch = opt[1];
-	if (ch < 'A' || ch >= 'A' + _GROUP_TOTAL_)
+	if (ch < 'A' || ch >= 'A' + GROUP_TOTAL_)
 		return -1;
 	panel_sort.group = ch - 'A';
-	return opt[2] == '\0' ? 0 : -1;
+	ch = opt[2];
+	if (ch == '\0')
+		return 0;		/* up to 4.6.5 */
+	if (ch < 'A' || ch >= 'A' + HIDE_TOTAL_)
+		return -1;
+	panel_sort.hide = ch - 'A';
+	return opt[3] == '\0' ? 0 : -1;
 }
 
 /* compare reversed strings */
