@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -20,6 +20,7 @@
 #include <stdlib.h>			/* setenv() */
 #include <string.h>			/* strcmp() */
 #include <unistd.h>			/* getpid() */
+#include "curses.h"			/* NCURSES_MOUSE_VERSION */
 
 #include "bookmarks.h"		/* bm_initialize() */
 #include "cfg.h"			/* cfg_initialize() */
@@ -57,7 +58,7 @@ static EXTRA_LINE el_bm[2] = {
 	{ L"Cancel",L"Changes will be discarded",0,cx_bm_revert }
 };
 static EXTRA_LINE el_cfg[3] = {
-	{ L"Cancel",L"Revert to the original state",MODE_SPECIAL_RETURN,0 },
+	{ L"Cancel",L"Changes will be discarded",MODE_SPECIAL_RETURN,0 },
 	{ L"Apply",L"Use the new configuration in this session",
 	  MODE_SPECIAL_RETURN,cx_cfg_apply },
 	{ L"Apply+Save",L"Save the configuration to disk",
@@ -74,6 +75,9 @@ static EXTRA_LINE el_group[2] = {
 	{ 0,0,MODE_SPECIAL_RETURN,0 },
 	{ L"Switch to user data (alt-U)",0,MODE_USER,0 }
 };
+static EXTRA_LINE el_sort[1] = {
+	{ L"Cancel",L"Changes will be discarded",MODE_SPECIAL_RETURN,0 }
+};
 static EXTRA_LINE el_user[2] = {
 	{ 0,0,MODE_SPECIAL_RETURN,0 },
 	{ L"Switch to group data (alt-G)",0,MODE_GROUP,0 }
@@ -89,11 +93,11 @@ static PANEL_DESC pd_bm = { 0,0,0,
   -1 * ARRAY_SIZE(el_bm),PANEL_TYPE_BM,0,el_bm,&shared_filt,draw_line_bm };
 static PANEL_DESC pd_bm_edit = { 2,0,0,
   EL_EXIT,PANEL_TYPE_BM,0,el_exit,0,draw_line_bm_edit };
-static PANEL_DESC pd_cfg = { _CFG_TOTAL_,0,0,
+static PANEL_DESC pd_cfg = { CFG_TOTAL_,0,0,
   -1 * ARRAY_SIZE(el_cfg),PANEL_TYPE_CFG,0,el_cfg,0,draw_line_cfg };
 static PANEL_DESC pd_cfg_menu = { 0,0,0,
   0,PANEL_TYPE_CFG_MENU,0,0,0,draw_line_cfg_menu };
-static PANEL_DESC pd_cmp = { 1 + _CMP_TOTAL_,0,0,
+static PANEL_DESC pd_cmp = { 1 + CMP_TOTAL_,0,0,
   EL_EXIT,PANEL_TYPE_CMP,0,el_exit,0,draw_line_cmp };
 static PANEL_DESC pd_cmp_sum = {  0,0,0,
   EL_EXIT,PANEL_TYPE_CMP_SUM,0,el_exit,0,draw_line_cmp_sum };
@@ -103,7 +107,7 @@ static PANEL_DESC pd_dir = { 0,0,0,
   -1 * ARRAY_SIZE(el_dir),PANEL_TYPE_DIR,0,el_dir,&shared_filt,draw_line_dir };
 static PANEL_DESC pd_dir_split = { 0,0,0,
   -1 * ARRAY_SIZE(el_dir_split),PANEL_TYPE_DIR_SPLIT,0,el_dir_split,0,draw_line_dir_split };
-static PANEL_DESC pd_fopt = { _FOPT_TOTAL_,0,0,
+static PANEL_DESC pd_fopt = { FOPT_TOTAL_,0,0,
   EL_EXIT,PANEL_TYPE_FOPT,0,el_exit,0,draw_line_fopt };
 static PANEL_DESC pd_grp = { 0,0,0,
   -1 * ARRAY_SIZE(el_group),PANEL_TYPE_GROUP,0,el_group,&shared_filt,draw_line_group };
@@ -115,12 +119,14 @@ static PANEL_DESC pd_log = { 0,0,0,
   EL_EXIT,PANEL_TYPE_LOG,0,el_exit,&log_filt,draw_line_log };
 static PANEL_DESC pd_mainmenu = /* 22 items in this menu */ { 22,EL_EXIT,EL_EXIT,
   EL_EXIT,PANEL_TYPE_MAINMENU,0,el_exit,0,draw_line_mainmenu };
-static PANEL_DESC pd_notif = { _NOTIF_TOTAL_,0,0,	
+static PANEL_DESC pd_notif = { NOTIF_TOTAL_,0,0,	
   EL_EXIT,PANEL_TYPE_NOTIF,0,el_exit,0,draw_line_notif };
 static PANEL_DESC pd_paste = /* 15 items in this menu */ { 15,EL_EXIT,EL_EXIT,
   EL_EXIT,PANEL_TYPE_PASTE,0,el_exit,0,draw_line_paste };
-static PANEL_DESC pd_sort = /* 14 items in this menu */ { 14,0,0,
-  EL_EXIT,PANEL_TYPE_SORT,0,el_exit,0,draw_line_sort };
+static PANEL_DESC pd_preview = /* 15 items in this menu */ { 0,0,0,
+  0,PANEL_TYPE_PREVIEW,0,0,0,draw_line_preview };
+static PANEL_DESC pd_sort = /* 18 items in this menu */ { 18,0,0,
+  EL_EXIT,PANEL_TYPE_SORT,0,el_sort,0,draw_line_sort };
 static PANEL_DESC pd_usr = { 0,0,0,
   -1 * ARRAY_SIZE(el_user),PANEL_TYPE_USER,0,el_user,&shared_filt,draw_line_user };
 PANEL_DESC *panel = 0;
@@ -141,7 +147,8 @@ PANEL_LOG panel_log = { &pd_log };
 PANEL_MENU panel_mainmenu = { &pd_mainmenu };
 PANEL_NOTIF panel_notif = { &pd_notif };
 PANEL_PASTE panel_paste = { &pd_paste };
-PANEL_SORT panel_sort =  { &pd_sort, GROUP_DSP, SORT_NAME };
+PANEL_PREVIEW panel_preview = { &pd_preview };
+PANEL_SORT panel_sort =  { &pd_sort, GROUP_DSP, SORT_NAME_NUM, HIDE_NEVER };
 PANEL_USER panel_user = { &pd_usr };
 PANEL_FILE *ppanel_file;
 
@@ -156,7 +163,7 @@ KBD_INPUT kinp;
 TEXTLINE *textline = 0, line_cmd, line_tmp, line_dir, line_inschar;
 CODE next_mode;
 volatile FLAG ctrlc_flag;
-const void *pcfg[_CFG_TOTAL_];
+const void *pcfg[CFG_TOTAL_];
 
 int
 main(int argc, char *argv[])
@@ -187,20 +194,25 @@ main(int argc, char *argv[])
 	}
 	if (version)
 		puts(
-		  "\nCLEX File Manager " VERSION CLEXRELEASE "\n"
+		  "\nCLEX File Manager " VERSION "\n"
 		  "  compiled with POSIX job control: "
 #ifdef _POSIX_JOB_CONTROL
-  "yes"
+  "yes\n"
 #else
-  "no"
+  "no\n"
 #endif
-		  "\n"
-		  "\nCopyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>"
+		  "  mouse interface: "
+#if NCURSES_MOUSE_VERSION >= 2
+  "ncurses\n"
+#else
+  "xterm\n"
+#endif
+		  "\nCopyright (C) 2001-2022 Vlado Potisk"
 		  "\n\n"
 		  "This is free software distributed without any warranty.\n"
 		  "See the GNU General Public License for more details.\n"
 		  "\n"
-		  "Project homepage is http://www.clex.sk");
+		  "Project homepage is https://github.com/xitop/clex");
 	if (help)
 		printf(
 		  "\nUsage: %s [OPTIONS]\n\n"
@@ -212,7 +224,7 @@ main(int argc, char *argv[])
 		exit(EXIT_SUCCESS);	/* no cleanup necessary at this stage */
 
 	/* real start */
-	puts("\n\n\nStarting CLEX " VERSION CLEXRELEASE "\n");
+	puts("\n\n\nStarting CLEX " VERSION "\n");
 
 	/* initialize program data */
 	clex_data.umask = umask(0777);
@@ -231,9 +243,9 @@ main(int argc, char *argv[])
 	userdata_initialize();	/* required by cfg_initialize */
 	cfg_initialize();
 	opt_initialize();
+	bm_initialize();
 
 	/* initialize the rest, the order is not important (except when noted) */
-	bm_initialize();
 	compl_initialize();
 	dir_initialize();
 	files_initialize();
