@@ -2,13 +2,13 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2022 Vlado Potisk
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
  * which accompanies the CLEX File Manager package.
  *
- * CLEX can be downloaded from http://www.clex.sk
+ * CLEX can be downloaded from https://github.com/xitop/clex
  *
  */
 
@@ -410,7 +410,7 @@ userdata_refresh(void)
 		reloaded = 1;
 	}
 
-	stat_ok = stat("/etc/group",&st) == 0; 
+	stat_ok = stat("/etc/group",&st) == 0;
 	if (!stat_ok || st.st_mtime >= gtable.timestamp
 	  || st.st_dev != gtable.device || st.st_ino != gtable.inode
       || now > gtable.timestamp + EXPIRATION) {
@@ -635,6 +635,7 @@ void
 user_panel_data(void)
 {
 	int i, j;
+	size_t len;
 	const wchar_t *login, *gecos;
 	uid_t curs;
 
@@ -642,6 +643,7 @@ user_panel_data(void)
 	if (panel_user.pd->filtering)
 		match_substr_set(panel_user.pd->filter->line);
 
+	panel_user.maxlen = 0;
 	for (i = j = 0; i < utable.cnt; i++) {
 		if (curs == utable.by_uid[i]->uid)
 			panel_user.pd->curs = j;
@@ -651,6 +653,9 @@ user_panel_data(void)
 			continue;
 		panel_user.users[j].uid = utable.by_uid[i]->uid;
 		panel_user.users[j].login = login;
+		len = wcslen(login);
+		if (len > panel_user.maxlen)
+			panel_user.maxlen = len;
 		panel_user.users[j++].gecos = gecos;
 	}
 
